@@ -8,10 +8,9 @@
 #include <sys/mman.h>
 #include <memory>
 #include <fcitx-utils/utf8.h>
-#include "fcitx/misc_p.h"
 #include "appmonitor.h"
-#include "plasmaappmonitor.h"
 #include "virtualinputcontext.h"
+#include "wayland-text-input-unstable-v1-client-protocol.h"
 #include "waylandim.h"
 
 #ifdef __linux__
@@ -48,9 +47,8 @@ static inline unsigned int waylandFormat(TextFormatFlags flags) {
 WaylandIMServer::WaylandIMServer(wl_display *display, FocusGroup *group,
                                  const std::string &name,
                                  WaylandIMModule *waylandim)
-    : group_(group), name_(name), parent_(waylandim), inputMethodV1_(nullptr),
-      display_(
-          static_cast<wayland::Display *>(wl_display_get_user_data(display))) {
+    : WaylandIMServerBase(display, group, name, waylandim),
+      inputMethodV1_(nullptr) {
     display_->requestGlobals<wayland::ZwpInputMethodV1>();
     globalConn_ = display_->globalCreated().connect(
         [this](const std::string &interface, const std::shared_ptr<void> &) {
@@ -90,7 +88,6 @@ void WaylandIMServer::init() {
                 WAYLANDIM_DEBUG() << "DEACTIVATE " << ic;
                 deactivate(ic);
             });
-        display_->flush();
     }
 }
 
@@ -108,6 +105,14 @@ void WaylandIMServer::deactivate(wayland::ZwpInputMethodContextV1 *id) {
     }
 }
 
+bool WaylandIMServer::hasKeyboardGrab() const {
+    if (auto *globalIc =
+            static_cast<WaylandIMInputContextV1 *>(globalIc_.get())) {
+        return globalIc->hasKeyboardGrab();
+    }
+    return false;
+}
+
 WaylandIMInputContextV1::WaylandIMInputContextV1(
     InputContextManager &inputContextManager, WaylandIMServer *server)
     : VirtualInputContextGlue(inputContextManager), server_(server) {
@@ -166,7 +171,6 @@ void WaylandIMInputContextV1::activate(wayland::ZwpInputMethodContextV1 *ic) {
         repeatInfoCallback(rate, delay);
     });
     repeatInfoCallback(repeatRate_, repeatDelay_);
-    server_->display_->sync();
     wl_array array;
     wl_array_init(&array);
     constexpr char data[] = "Shift\0Control\0Mod1\0Mod4";
@@ -191,7 +195,6 @@ void WaylandIMInputContextV1::deactivate(wayland::ZwpInputMethodContextV1 *ic) {
         server_->instance()->clearXkbStateMask(server_->group()->display());
 
         timeEvent_->setEnabled(false);
-        server_->display_->sync();
         focusOutWrapper();
     } else {
         // This should not happen, but just in case.
@@ -210,17 +213,14 @@ void WaylandIMInputContextV1::repeat() {
         Key(repeatSym_, server_->modifiers_ | KeyState::Repeat, repeatKey_ + 8),
         false, repeatTime_);
 
-    sendKeyToVK(repeatTime_, event.rawKey().code() - 8,
-                WL_KEYBOARD_KEY_STATE_RELEASED);
+    sendKeyToVK(repeatTime_, event.rawKey(), WL_KEYBOARD_KEY_STATE_RELEASED);
     if (!ic->keyEvent(event)) {
-        sendKeyToVK(repeatTime_, event.rawKey().code() - 8,
-                    WL_KEYBOARD_KEY_STATE_PRESSED);
+        sendKeyToVK(repeatTime_, event.rawKey(), WL_KEYBOARD_KEY_STATE_PRESSED);
     }
 
     uint64_t interval = 1000000 / repeatRate_;
     timeEvent_->setTime(timeEvent_->time() + interval);
     timeEvent_->setOneShot();
-    server_->display_->flush();
 }
 
 void WaylandIMInputContextV1::surroundingTextCallback(const char *text,
@@ -431,8 +431,8 @@ void WaylandIMInputContextV1::keymapCallback(uint32_t format, int32_t fd,
     server_->parent_->wayland()->call<IWaylandModule::reloadXkbOption>();
 }
 
-void WaylandIMInputContextV1::keyCallback(uint32_t serial, uint32_t time,
-                                          uint32_t key, uint32_t state) {
+void WaylandIMInputContextV1::keyCallback(uint32_t, uint32_t time, uint32_t key,
+                                          uint32_t state) {
     time_ = time;
     if (!server_->state_) {
         return;
@@ -469,9 +469,20 @@ void WaylandIMInputContextV1::keyCallback(uint32_t serial, uint32_t time,
     WAYLANDIM_DEBUG() << event.key().toString()
                       << " IsRelease=" << event.isRelease();
     if (!ic->keyEvent(event)) {
-        ic_->key(serial, time, key, state);
+        sendKeyToVK(time, event.rawKey(), state);
+    }
+
+    // This means our engine is being too slow, this is usually transient (e.g.
+    // cold start up due to data loading, high CPU usage etc).
+    // To avoid an undesired repetition, reset the delay the next interval so we
+    // can handle the release first.
+    if (timeEvent_->time() < now(timeEvent_->clock()) &&
+        timeEvent_->isOneShot()) {
+        WAYLANDIM_DEBUG() << "Engine handling speed can not keep up with key "
+                             "repetition rate.";
+        timeEvent_->setNextInterval(
+            std::min(1000, repeatDelay_ * 1000 - repeatHackDelay));
     }
-    server_->display_->flush();
 }
 void WaylandIMInputContextV1::modifiersCallback(uint32_t serial,
                                                 uint32_t mods_depressed,
@@ -542,13 +553,17 @@ void WaylandIMInputContextV1::sendKey(uint32_t time, uint32_t sym,
     ic_->keysym(serial_, time, sym, state, modifiers);
 }
 
-void WaylandIMInputContextV1::sendKeyToVK(uint32_t time, uint32_t key,
-                                          uint32_t state) {
+void WaylandIMInputContextV1::sendKeyToVK(uint32_t time, const Key &key,
+                                          uint32_t state) const {
     if (!ic_) {
         return;
     }
-    ic_->key(serial_, time, key, state);
-    server_->display_->flush();
+
+    if (auto text = server_->mayCommitAsText(key, state)) {
+        ic_->commitString(serial_, text->data());
+    } else {
+        ic_->key(serial_, time, key.code() - 8, state);
+    }
 }
 
 void WaylandIMInputContextV1::updatePreeditDelegate(InputContext *ic) const {
@@ -565,7 +580,9 @@ void WaylandIMInputContextV1::updatePreeditDelegate(InputContext *ic) const {
         }
     }
 
-    ic_->preeditCursor(preedit.cursor());
+    // Though negative cursor is allowed by protocol, we just don't use it.
+    ic_->preeditCursor(preedit.cursor() >= 0 ? preedit.cursor()
+                                             : preedit.textLength());
     unsigned int index = 0;
     for (int i = 0, e = preedit.size(); i < e; i++) {
         if (!preedit.stringAt(i).empty()) {
