@@ -6,6 +6,7 @@
  */
 #include "waylandimserverv2.h"
 #include <sys/mman.h>
+#include "fcitx-utils/keysymgen.h"
 #include "fcitx-utils/unixfd.h"
 #include "fcitx-utils/utf8.h"
 #include "fcitx/inputcontext.h"
@@ -23,9 +24,8 @@ constexpr CapabilityFlags baseFlags{CapabilityFlag::Preedit,
 WaylandIMServerV2::WaylandIMServerV2(wl_display *display, FocusGroup *group,
                                      const std::string &name,
                                      WaylandIMModule *waylandim)
-    : group_(group), name_(name), parent_(waylandim),
-      inputMethodManagerV2_(nullptr), display_(static_cast<wayland::Display *>(
-                                          wl_display_get_user_data(display))) {
+    : WaylandIMServerBase(display, group, name, waylandim),
+      inputMethodManagerV2_(nullptr) {
     display_->requestGlobals<wayland::ZwpInputMethodManagerV2>();
     display_->requestGlobals<wayland::ZwpVirtualKeyboardManagerV1>();
     display_->requestGlobals<wayland::WlSeat>();
@@ -70,6 +70,13 @@ InputContextManager &WaylandIMServerV2::inputContextManager() {
 
 Instance *WaylandIMServerV2::instance() { return parent_->instance(); }
 
+bool WaylandIMServerV2::hasKeyboardGrab() const {
+    return std::any_of(icMap_.begin(), icMap_.end(),
+                       [](const decltype(icMap_)::value_type &ic) {
+                           return ic.second && ic.second->hasKeyboardGrab();
+                       });
+}
+
 void WaylandIMServerV2::init() {
     if (init_) {
         return;
@@ -81,8 +88,6 @@ void WaylandIMServerV2::init() {
     }
     WAYLANDIM_DEBUG() << "INIT IM V2";
     refreshSeat();
-
-    display_->flush();
 }
 
 void WaylandIMServerV2::refreshSeat() {
@@ -150,11 +155,13 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
                     // If last key to vk is press, send a release.
                     while (!pressedVKKey_.empty()) {
                         auto [vkkey, vktime] = *pressedVKKey_.begin();
-                        sendKeyToVK(vktime, vkkey,
+                        // This is key release, so we don't need real sym and
+                        // states.
+                        sendKeyToVK(vktime,
+                                    Key(FcitxKey_None, KeyStates(), vkkey + 8),
                                     WL_KEYBOARD_KEY_STATE_RELEASED);
                     }
                     vk_->modifiers(0, 0, 0, 0);
-                    server_->display_->sync();
                 }
                 focusOutWrapper();
             }
@@ -192,7 +199,6 @@ WaylandIMInputContextV2::WaylandIMInputContextV2(
                     });
                 repeatInfoCallback(repeatRate_, repeatDelay_);
                 focusInWrapper();
-                server_->display_->sync();
             }
         }
     });
@@ -234,11 +240,9 @@ void WaylandIMInputContextV2::repeat() {
         ic,
         Key(repeatSym_, server_->modifiers_ | KeyState::Repeat, repeatKey_ + 8),
         false, repeatTime_);
-    sendKeyToVK(repeatTime_, event.rawKey().code() - 8,
-                WL_KEYBOARD_KEY_STATE_RELEASED);
+    sendKeyToVK(repeatTime_, event.rawKey(), WL_KEYBOARD_KEY_STATE_RELEASED);
     if (!ic->keyEvent(event)) {
-        sendKeyToVK(repeatTime_, event.rawKey().code() - 8,
-                    WL_KEYBOARD_KEY_STATE_PRESSED);
+        sendKeyToVK(repeatTime_, event.rawKey(), WL_KEYBOARD_KEY_STATE_PRESSED);
     }
     uint64_t interval = 1000000 / repeatRate_;
     timeEvent_->setTime(timeEvent_->time() + interval);
@@ -413,12 +417,6 @@ void WaylandIMInputContextV2::keymapCallback(uint32_t format, int32_t fd,
         1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Mod4");
     server_->stateMask_.mod5_mask =
         1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Mod5");
-    server_->stateMask_.super_mask =
-        1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Super");
-    server_->stateMask_.hyper_mask =
-        1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Hyper");
-    server_->stateMask_.meta_mask =
-        1 << xkb_keymap_mod_get_index(server_->keymap_.get(), "Meta");
 
     if (keymapChanged) {
         vk_->keymap(format, scopeFD.fd(), size);
@@ -467,11 +465,22 @@ void WaylandIMInputContextV2::keyCallback(uint32_t, uint32_t time, uint32_t key,
     WAYLANDIM_DEBUG() << event.key().toString()
                       << " IsRelease=" << event.isRelease();
     if (!ic->keyEvent(event)) {
-        sendKeyToVK(time, event.rawKey().code() - 8,
+        sendKeyToVK(time, event.rawKey(),
                     event.isRelease() ? WL_KEYBOARD_KEY_STATE_RELEASED
                                       : WL_KEYBOARD_KEY_STATE_PRESSED);
     }
-    server_->display_->flush();
+
+    // This means our engine is being too slow, this is usually transient (e.g.
+    // cold start up due to data loading, high CPU usage etc).
+    // To avoid an undesired repetition, reset the delay the next interval so we
+    // can handle the release first.
+    if (timeEvent_->time() < now(timeEvent_->clock()) &&
+        timeEvent_->isOneShot()) {
+        WAYLANDIM_DEBUG() << "Engine handling speed can not keep up with key "
+                             "repetition rate.";
+        timeEvent_->setNextInterval(
+            std::min(1000, repeatDelay_ * 1000 - repeatHackDelay));
+    }
 }
 void WaylandIMInputContextV2::modifiersCallback(uint32_t,
                                                 uint32_t mods_depressed,
@@ -508,24 +517,15 @@ void WaylandIMInputContextV2::modifiersCallback(uint32_t,
     if (mask & server_->stateMask_.mod2_mask) {
         server_->modifiers_ |= KeyState::NumLock;
     }
-    if (mask & server_->stateMask_.super_mask) {
-        server_->modifiers_ |= KeyState::Super;
-    }
     if (mask & server_->stateMask_.mod4_mask) {
         server_->modifiers_ |= KeyState::Super;
     }
-    if (mask & server_->stateMask_.hyper_mask) {
-        server_->modifiers_ |= KeyState::Hyper;
-    }
     if (mask & server_->stateMask_.mod3_mask) {
-        server_->modifiers_ |= KeyState::Hyper;
+        server_->modifiers_ |= KeyState::Mod3;
     }
     if (mask & server_->stateMask_.mod5_mask) {
         server_->modifiers_ |= KeyState::Mod5;
     }
-    if (mask & server_->stateMask_.meta_mask) {
-        server_->modifiers_ |= KeyState::Meta;
-    }
 
     if (vkReady_) {
         vk_->modifiers(mods_depressed, mods_latched, mods_locked, group);
@@ -537,18 +537,23 @@ void WaylandIMInputContextV2::repeatInfoCallback(int32_t rate, int32_t delay) {
     repeatDelay_ = delay;
 }
 
-void WaylandIMInputContextV2::sendKeyToVK(uint32_t time, uint32_t key,
+void WaylandIMInputContextV2::sendKeyToVK(uint32_t time, const Key &key,
                                           uint32_t state) const {
     if (!vkReady_) {
         return;
     }
-    // Erase old to ensure order, and released ones can the be removed.
-    pressedVKKey_.erase(key);
-    if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
-        pressedVKKey_[key] = time;
+
+    uint32_t code = key.code() - 8;
+    if (auto text = server_->mayCommitAsText(key, state)) {
+        commitStringDelegate(this, *text);
+        return;
     }
-    vk_->key(time, key, state);
-    server_->display_->flush();
+    // Erase old to ensure order, and released ones can the be removed.
+    pressedVKKey_.erase(code);
+    if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
+        pressedVKKey_[code] = time;
+    }
+    vk_->key(time, code, state);
 }
 
 void WaylandIMInputContextV2::forwardKeyDelegate(
@@ -568,11 +573,14 @@ void WaylandIMInputContextV2::forwardKeyDelegate(
             }
         }
     }
-    sendKeyToVK(time_, code - 8,
+
+    Key keyWithCode(key.rawKey().sym(), key.rawKey().states(), code);
+
+    sendKeyToVK(time_, keyWithCode,
                 key.isRelease() ? WL_KEYBOARD_KEY_STATE_RELEASED
                                 : WL_KEYBOARD_KEY_STATE_PRESSED);
     if (!key.isRelease()) {
-        sendKeyToVK(time_, code - 8, WL_KEYBOARD_KEY_STATE_RELEASED);
+        sendKeyToVK(time_, keyWithCode, WL_KEYBOARD_KEY_STATE_RELEASED);
     }
 }
 
@@ -613,6 +621,9 @@ void WaylandIMInputContextV2::updatePreeditDelegate(InputContext *ic) const {
     }
 
     if (preedit.textLength()) {
+        if (cursorStart < 0) {
+            cursorStart = cursorEnd = preedit.textLength();
+        }
         ic_->setPreeditString(preedit.toString().data(), cursorStart,
                               cursorEnd);
     }
