@@ -16,6 +16,7 @@
 #include "fcitx/inputcontext.h"
 #include "display.h"
 #include "virtualinputcontext.h"
+#include "waylandimserverbase.h"
 #include "zwp_input_method_keyboard_grab_v2.h"
 #include "zwp_input_method_manager_v2.h"
 #include "zwp_input_method_v2.h"
@@ -26,7 +27,7 @@ namespace fcitx {
 class WaylandIMModule;
 class WaylandIMInputContextV2;
 
-class WaylandIMServerV2 {
+class WaylandIMServerV2 : public WaylandIMServerBase {
     friend class WaylandIMInputContextV2;
 
 public:
@@ -39,8 +40,6 @@ public:
 
     void init();
     void refreshSeat();
-    void activate(wayland::ZwpInputMethodV2 *id);
-    void deactivate(wayland::ZwpInputMethodV2 *id);
     void add(WaylandIMInputContextV2 *ic, wayland::WlSeat *seat);
     void remove(wayland::WlSeat *seat);
     Instance *instance();
@@ -48,21 +47,16 @@ public:
     auto *xkbState() { return state_.get(); }
     auto *inputMethodManagerV2() { return inputMethodManagerV2_.get(); }
 
+    bool hasKeyboardGrab() const;
+
 private:
-    FocusGroup *group_;
-    std::string name_;
     bool init_ = false;
-    WaylandIMModule *parent_;
     std::shared_ptr<wayland::ZwpInputMethodManagerV2> inputMethodManagerV2_;
     std::shared_ptr<wayland::ZwpVirtualKeyboardManagerV1>
         virtualKeyboardManagerV1_;
 
-    UniqueCPtr<struct xkb_context, xkb_context_unref> context_;
     std::vector<char> keymapData_;
-    UniqueCPtr<struct xkb_keymap, xkb_keymap_unref> keymap_;
-    UniqueCPtr<struct xkb_state, xkb_state_unref> state_;
 
-    wayland::Display *display_;
     ScopedConnection globalConn_;
 
     struct StateMask {
@@ -74,13 +68,8 @@ private:
         uint32_t mod3_mask = 0;
         uint32_t mod4_mask = 0;
         uint32_t mod5_mask = 0;
-        uint32_t super_mask = 0;
-        uint32_t hyper_mask = 0;
-        uint32_t meta_mask = 0;
     } stateMask_;
 
-    KeyStates modifiers_;
-
     std::unordered_map<wayland::WlSeat *, WaylandIMInputContextV2 *> icMap_;
 };
 
@@ -96,8 +85,10 @@ public:
 
     auto inputMethodV2() { return ic_.get(); }
 
+    bool hasKeyboardGrab() const { return keyboardGrab_.get(); }
+
 protected:
-    void commitStringDelegate(InputContext *,
+    void commitStringDelegate(const InputContext *,
                               const std::string &text) const override {
         if (!ic_) {
             return;
@@ -127,7 +118,7 @@ private:
                            uint32_t mods_latched, uint32_t mods_locked,
                            uint32_t group);
     void repeatInfoCallback(int32_t rate, int32_t delay);
-    void sendKeyToVK(uint32_t time, uint32_t key, uint32_t state) const;
+    void sendKeyToVK(uint32_t time, const Key &key, uint32_t state) const;
 
     WaylandIMServerV2 *server_;
     std::shared_ptr<wayland::WlSeat> seat_;
