@@ -17,6 +17,7 @@
 #include <type_traits>
 #include <unordered_map>
 #include <config.h>
+#include "endian_p.h"
 
 namespace fcitx {
 
@@ -29,6 +30,12 @@ decltype(&std::declval<M>().begin()->second) findValue(M &&m, K &&key) {
     return nullptr;
 }
 
+template <typename C, typename V>
+bool containerContains(C &&container, V &&value) {
+    return std::find(std::begin(container), std::end(container), value) !=
+           std::end(container);
+}
+
 template <typename T>
 class OrderedSet {
     typedef std::list<T> OrderList;
@@ -299,6 +306,14 @@ static inline bool checkBoolEnvVar(const char *name) {
     return value;
 }
 
+template <typename T>
+static inline uint32_t FromLittleEndian32(const T *d) {
+    const uint8_t *data = reinterpret_cast<const uint8_t *>(d);
+    uint32_t t;
+    memcpy(&t, data, sizeof(t));
+    return le32toh(t);
+}
+
 } // namespace fcitx
 
 #endif // _FCITX_UTILS_MISC_P_H_
