@@ -34,4 +34,15 @@ AddonFunctionAdaptorBase *AddonInstance::findCall(const std::string &name) {
     }
     return iter->second;
 }
+
+void AddonInstance::setCanRestart(bool canRestart) {
+    FCITX_D();
+    d->canRestart_ = canRestart;
+}
+
+bool AddonInstance::canRestart() const {
+    FCITX_D();
+    return d->canRestart_;
+}
+
 } // namespace fcitx
