@@ -8,6 +8,7 @@
 #include "globalconfig.h"
 #include "fcitx-config/configuration.h"
 #include "fcitx-config/iniparser.h"
+#include "fcitx-config/option.h"
 #include "fcitx-utils/i18n.h"
 #include "config.h"
 #include "inputcontextmanager.h"
@@ -138,7 +139,8 @@ FCITX_CONFIGURATION(
     OptionWithAnnotation<PropertyPropagatePolicy,
                          PropertyPropagatePolicyI18NAnnotation>
         shareState{this, "ShareInputState", _("Share Input State"),
-                   PropertyPropagatePolicy::No};
+                   isAndroid() ? PropertyPropagatePolicy::All
+                               : PropertyPropagatePolicy::No};
     Option<bool> preeditEnabledByDefault{this, "PreeditEnabledByDefault",
                                          _("Show preedit in application"),
                                          true};
@@ -178,7 +180,22 @@ FCITX_CONFIGURATION(
         this, "DisabledAddons", "Force Disabled Addons"};
     HiddenOption<bool> preloadInputMethod{
         this, "PreloadInputMethod",
-        "Preload input method to be used by default", true};);
+        "Preload input method to be used by default", true};
+    Option<bool> allowInputMethodForPassword{
+        this, "AllowInputMethodForPassword",
+        _("Allow input method in the password field"), false};
+    Option<bool> showPreeditForPassword{
+        this, "ShowPreeditForPassword",
+        _("Show preedit text when typing password"), false};
+    Option<int, IntConstrain, DefaultMarshaller<int>, ToolTipAnnotation>
+        autoSavePeriod{this,
+                       "AutoSavePeriod",
+                       _("Interval of saving user data in minutes"),
+                       30,
+                       IntConstrain(0, 1440),
+                       {},
+                       {_("If value is 0, the user data may only be saved when "
+                          "fcitx quits (e.g. logout).")}};);
 
 FCITX_CONFIGURATION(GlobalConfig,
                     Option<HotkeyConfig> hotkey{this, "Hotkey", _("Hotkey")};
@@ -357,6 +374,21 @@ bool GlobalConfig::preloadInputMethod() const {
     return *d->behavior->preloadInputMethod;
 }
 
+bool GlobalConfig::allowInputMethodForPassword() const {
+    FCITX_D();
+    return *d->behavior->allowInputMethodForPassword;
+}
+
+bool GlobalConfig::showPreeditForPassword() const {
+    FCITX_D();
+    return *d->behavior->showPreeditForPassword;
+}
+
+int GlobalConfig::autoSavePeriod() const {
+    FCITX_D();
+    return *d->behavior->autoSavePeriod;
+}
+
 const Configuration &GlobalConfig::config() const {
     FCITX_D();
     return *d;
