@@ -10,7 +10,10 @@
 #include <chrono>
 #include <exception>
 #include <regex>
+#include <stdexcept>
+#include <string>
 #include "fcitx-utils/event.h"
+#include "fcitx-utils/utf8.h"
 #include "focusgroup.h"
 #include "inputcontext_p.h"
 #include "inputcontextmanager.h"
@@ -420,6 +423,22 @@ void InputContext::commitString(const std::string &text) {
     }
 }
 
+void InputContext::commitStringWithCursor(const std::string &text,
+                                          size_t cursor) {
+    FCITX_D();
+    if (cursor > utf8::length(text)) {
+        throw std::invalid_argument(text);
+    }
+
+    if (auto *instance = d->manager_.instance()) {
+        auto newString = instance->commitFilter(this, text);
+        d->pushEvent<CommitStringWithCursorEvent>(std::move(newString), cursor,
+                                                  this);
+    } else {
+        d->pushEvent<CommitStringWithCursorEvent>(text, cursor, this);
+    }
+}
+
 void InputContext::deleteSurroundingText(int offset, unsigned int size) {
     deleteSurroundingTextImpl(offset, size);
 }
@@ -471,6 +490,8 @@ const StatusArea &InputContext::statusArea() const {
 
 void InputContext::updateClientSideUIImpl() {}
 
+InputContextV2::~InputContextV2() = default;
+
 InputContextEventBlocker::InputContextEventBlocker(InputContext *inputContext)
     : inputContext_(inputContext->watch()) {
     inputContext->setBlockEventToClient(true);
