@@ -11,10 +11,12 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <stdexcept>
+#include <unordered_map>
 #include <utility>
 #include <fmt/format.h>
 #include <getopt.h>
 #include "fcitx-config/iniparser.h"
+#include "fcitx-utils/capabilityflags.h"
 #include "fcitx-utils/event.h"
 #include "fcitx-utils/i18n.h"
 #include "fcitx-utils/log.h"
@@ -52,8 +54,8 @@ namespace fcitx {
 
 namespace {
 
-constexpr uint64_t AutoSavePeriod = 1800ull * 1000000ull; // 30 minutes
-constexpr uint64_t AutoSaveIdleTime = 60ull * 1000000ull; // 1 minutes
+constexpr uint64_t AutoSaveMinInUsecs = 60ull * 1000000ull; // 30 minutes
+constexpr uint64_t AutoSaveIdleTime = 60ull * 1000000ull;   // 1 minutes
 
 FCITX_CONFIGURATION(DefaultInputMethod,
                     Option<std::vector<std::string>> defaultInputMethods{
@@ -78,7 +80,7 @@ void initAsDaemon() {
     if (fork() > 0) {
         exit(0);
     }
-    chdir("/");
+    (void)chdir("/");
 
     signal(SIGINT, oldint);
     signal(SIGHUP, oldhup);
@@ -136,6 +138,9 @@ void InstanceArgument::printUsage() const {
         << "  -k, --keep\t\t\tKeep running even the main display is "
            "disconnected.\n"
         << "  -r, --replace\t\t\tReplace the existing instance.\n"
+        << "  -o --option <option>\t\tPass the option to addons\n"
+        << "\t\t\t\t<option> is in format like:"
+        << "\t\t\t\tname1=opt1a:opt1b,name2=opt2a:opt2b... .\n"
         << "  -v, --version\t\t\tShow version and quit.\n"
         << "  -h, --help\t\t\tShow this help message and quit.\n";
 }
@@ -638,6 +643,7 @@ Instance::Instance(int argc, char **argv) {
     FCITX_D();
     d->arg_ = arg;
     d->addonManager_.setInstance(this);
+    d->addonManager_.setAddonOptions(arg.addonOptions_);
     d->icManager_.setInstance(this);
     d->connections_.emplace_back(
         d->imManager_.connect<InputMethodManager::CurrentGroupAboutToChange>(
@@ -904,7 +910,8 @@ Instance::Instance(int argc, char **argv) {
             FCITX_KEYTRACE() << "KeyEvent: " << keyEvent.key()
                              << " rawKey: " << keyEvent.rawKey()
                              << " origKey: " << keyEvent.origKey()
-                             << " Release:" << keyEvent.isRelease();
+                             << " Release:" << keyEvent.isRelease()
+                             << " keycode: " << keyEvent.origKey().code();
 
             if (keyEvent.isRelease()) {
                 return;
@@ -966,9 +973,10 @@ Instance::Instance(int argc, char **argv) {
                 if (auto *xkbState = inputState->customXkbState()) {
                     if (auto utf32 = xkb_state_key_get_utf32(
                             xkbState, keyEvent.key().code())) {
-                        // Ignore backspace, return, backspace, and delete.
+                        // Ignore newline, return, backspace, tab, and delete.
                         if (utf32 == '\n' || utf32 == '\b' || utf32 == '\r' ||
-                            utf32 == '\033' || utf32 == '\x7f') {
+                            utf32 == '\t' || utf32 == '\033' ||
+                            utf32 == '\x7f') {
                             return;
                         }
                         if (keyEvent.key().states().test(KeyState::Ctrl) ||
@@ -1160,7 +1168,7 @@ Instance::Instance(int argc, char **argv) {
     });
     d->uiUpdateEvent_->setEnabled(false);
     d->periodicalSave_ = d->eventLoop_.addTimeEvent(
-        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + AutoSavePeriod, 0,
+        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 1000000, AutoSaveIdleTime,
         [this, d](EventSourceTime *time, uint64_t) {
             if (exiting()) {
                 return true;
@@ -1179,10 +1187,14 @@ Instance::Instance(int argc, char **argv) {
             FCITX_INFO() << "Running autosave...";
             save();
             FCITX_INFO() << "End autosave";
-            time->setNextInterval(AutoSavePeriod);
-            time->setOneShot();
+            if (d->globalConfig_.autoSavePeriod() > 0) {
+                time->setNextInterval(d->globalConfig_.autoSavePeriod() *
+                                      AutoSaveMinInUsecs);
+                time->setOneShot();
+            }
             return true;
         });
+    d->periodicalSave_->setEnabled(false);
 }
 
 Instance::~Instance() {
@@ -1207,11 +1219,13 @@ void InstanceArgument::parseOption(int argc, char **argv) {
                                    {"replace", no_argument, nullptr, 'r'},
                                    {"version", no_argument, nullptr, 'v'},
                                    {"help", no_argument, nullptr, 'h'},
+                                   {"option", required_argument, nullptr, 'o'},
                                    {nullptr, 0, 0, 0}};
 
     int optionIndex = 0;
     int c;
-    while ((c = getopt_long(argc, argv, "ru:dDs:hv", longOptions,
+    std::string addonOptionString;
+    while ((c = getopt_long(argc, argv, "ru:dDs:hvo:", longOptions,
                             &optionIndex)) != EOF) {
         switch (c) {
         case 0: {
@@ -1257,6 +1271,9 @@ void InstanceArgument::parseOption(int argc, char **argv) {
             quietQuit = true;
             printVersion();
             break;
+        case 'o':
+            addonOptionString = optarg;
+            break;
         default:
             quietQuit = true;
             printUsage();
@@ -1265,6 +1282,17 @@ void InstanceArgument::parseOption(int argc, char **argv) {
             break;
         }
     }
+
+    std::unordered_map<std::string, std::vector<std::string>> addonOptions;
+    for (const std::string_view item :
+         stringutils::split(addonOptionString, ",")) {
+        auto tokens = stringutils::split(item, "=");
+        if (tokens.size() != 2) {
+            continue;
+        }
+        addonOptions[tokens[0]] = stringutils::split(tokens[1], ":");
+    }
+    addonOptions_ = std::move(addonOptions);
 }
 
 void Instance::setSignalPipe(int fd) {
@@ -1366,15 +1394,16 @@ int Instance::exec() {
         return 0;
     }
     d->exit_ = false;
+    d->exitCode_ = 0;
     initialize();
     if (d->exit_) {
-        return 1;
+        return d->exitCode_;
     }
     d->running_ = true;
     auto r = eventLoop().exec();
     d->running_ = false;
 
-    return r ? 0 : 1;
+    return r ? d->exitCode_ : 1;
 }
 
 void Instance::setRunning(bool running) {
@@ -1437,6 +1466,25 @@ void Instance::setVirtualKeyboardFunctionMode(
     d->virtualKeyboardFunctionMode_ = mode;
 }
 
+void Instance::setBinaryMode() {
+    FCITX_D();
+    d->binaryMode_ = true;
+}
+
+bool Instance::canRestart() const {
+    FCITX_D();
+    const auto &addonNames = d->addonManager_.loadedAddonNames();
+    return d->binaryMode_ &&
+           std::all_of(addonNames.begin(), addonNames.end(),
+                       [d](const std::string &name) {
+                           auto addon = d->addonManager_.lookupAddon(name);
+                           if (!addon) {
+                               return true;
+                           }
+                           return addon->canRestart();
+                       });
+}
+
 InstancePrivate *Instance::privateData() {
     FCITX_D();
     return d;
@@ -1575,10 +1623,11 @@ std::string Instance::inputMethod(InputContext *ic) {
     }
 
     auto &group = d->imManager_.currentGroup();
-    if (ic->capabilityFlags().testAny(CapabilityFlags{
-            CapabilityFlag::Password, CapabilityFlag::Disable})) {
+    if (ic->capabilityFlags().test(CapabilityFlag::Disable) ||
+        (ic->capabilityFlags().test(CapabilityFlag::Password) &&
+         !d->globalConfig_.allowInputMethodForPassword())) {
         auto defaultLayout = group.defaultLayout();
-        auto defaultLayoutIM = fmt::format("keyboard-{}", defaultLayout);
+        auto defaultLayoutIM = stringutils::concat("keyboard-", defaultLayout);
         const auto *entry = d->imManager_.entry(defaultLayoutIM);
         if (!entry) {
             entry = d->imManager_.entry("keyboard-us");
@@ -1846,9 +1895,12 @@ void Instance::deactivate() {
     }
 }
 
-void Instance::exit() {
+void Instance::exit() { exit(0); }
+
+void Instance::exit(int exitCode) {
     FCITX_D();
     d->exit_ = true;
+    d->exitCode_ = exitCode;
     if (d->running_) {
         d->eventLoop_.exit();
     }
@@ -1902,6 +1954,14 @@ void Instance::reloadConfig() {
     if (d->running_) {
         postEvent(GlobalConfigReloadedEvent());
     }
+
+    if (d->globalConfig_.autoSavePeriod() <= 0) {
+        d->periodicalSave_->setEnabled(false);
+    } else {
+        d->periodicalSave_->setNextInterval(AutoSaveMinInUsecs *
+                                            d->globalConfig_.autoSavePeriod());
+        d->periodicalSave_->setOneShot();
+    }
 }
 
 void Instance::resetInputMethodList() {
@@ -1911,6 +1971,9 @@ void Instance::resetInputMethodList() {
 
 void Instance::restart() {
     FCITX_D();
+    if (!canRestart()) {
+        return;
+    }
     d->restart_ = true;
     exit();
 }
@@ -2193,6 +2256,7 @@ Text Instance::outputFilter(InputContext *inputContext, const Text &orig) {
     emit<Instance::OutputFilter>(inputContext, result);
     if ((&orig == &inputContext->inputPanel().clientPreedit() ||
          &orig == &inputContext->inputPanel().preedit()) &&
+        !globalConfig().showPreeditForPassword() &&
         inputContext->capabilityFlags().test(CapabilityFlag::Password)) {
         Text newText;
         for (int i = 0, e = result.size(); i < e; i++) {
