@@ -76,18 +76,6 @@ nthCandidateIgnorePlaceholder(const CandidateList &candidateList, int idx) {
         idx);
 }
 
-static inline const CandidateWord *
-nthBulkCandidateIgnorePlaceholder(const BulkCandidateList &bulkCandidateList,
-                                  int idx) {
-
-    return nthCandidateIgnorePlaceholder(
-        [&bulkCandidateList]() { return bulkCandidateList.totalSize(); },
-        [&bulkCandidateList](int idx) -> const CandidateWord & {
-            return bulkCandidateList.candidateFromAll(idx);
-        },
-        idx);
-}
-
 static inline std::string readFileContent(const std::string &file) {
     std::ifstream fin(file, std::ios::binary | std::ios::in);
     std::vector<char> buffer;
@@ -131,15 +119,25 @@ enum class DesktopType {
     LXDE,
     XFCE,
     DEEPIN,
+    UKUI,
+    Sway,
     Unknown
 };
 
 static inline DesktopType getDesktopType() {
     std::string desktop;
+    // new standard
     auto *desktopEnv = getenv("XDG_CURRENT_DESKTOP");
     if (desktopEnv) {
         desktop = desktopEnv;
     }
+    if (desktop.empty()) {
+        // old standard, guaranteed by display manager.
+        desktopEnv = getenv("DESKTOP_SESSION");
+        if (desktopEnv) {
+            desktop = desktopEnv;
+        }
+    }
 
     for (auto &c : desktop) {
         c = charutils::tolower(c);
@@ -172,6 +170,10 @@ static inline DesktopType getDesktopType() {
             return DesktopType::XFCE;
         } else if (desktop == "deepin") {
             return DesktopType::DEEPIN;
+        } else if (desktop == "ukui") {
+            return DesktopType::UKUI;
+        } else if (desktop == "sway") {
+            return DesktopType::Sway;
         }
     }
     return DesktopType::Unknown;
@@ -237,6 +239,10 @@ static inline bool isSingleKey(const Key &key) {
     return isSingleModifier(key) || !key.hasModifier();
 }
 
+inline void hash_combine(std::size_t &seed, std::size_t value) noexcept {
+    seed ^= value + 0x9e3779b9 + (seed << 6) + (seed >> 2);
+}
+
 } // namespace fcitx
 
 FCITX_DECLARE_LOG_CATEGORY(keyTrace);
