@@ -31,23 +31,22 @@ public:
     InputBuffer buffer_;
     Unicode *q_;
 
-    void reset(InputContext *ic) {
+    void reset(InputContext *inputContext) {
         mode_ = UnicodeMode::Off;
         buffer_.clear();
         buffer_.shrinkToFit();
-        ic->inputPanel().reset();
-        ic->updatePreedit();
-        ic->updateUserInterface(UserInterfaceComponent::InputPanel);
+        inputContext->inputPanel().reset();
+        inputContext->updatePreedit();
+        inputContext->updateUserInterface(UserInterfaceComponent::InputPanel);
     }
 };
 
 class UnicodeCandidateWord : public CandidateWord {
 public:
-    UnicodeCandidateWord(Unicode *q, int c) : q_(q) {
+    UnicodeCandidateWord(Unicode *q, uint32_t chr) : q_(q) {
         Text text;
-        text.append(utf8::UCS4ToUTF8(c));
-        text.append(" ");
-        text.append(q->data().name(c));
+        text.append(utf8::UCS4ToUTF8(chr));
+        text.append(stringutils::concat(" ", q->data().name(chr)));
         setText(std::move(text));
     }
 
@@ -139,8 +138,9 @@ Unicode::Unicode(Instance *instance)
 Unicode::~Unicode() {}
 
 bool Unicode::trigger(InputContext *inputContext) {
-    if (!data_.load())
+    if (!data_.load()) {
         return false;
+    }
     auto *state = inputContext->propertyFor(&factory_);
     state->mode_ = UnicodeMode::Search;
     updateUI(inputContext, true);
@@ -148,8 +148,9 @@ bool Unicode::trigger(InputContext *inputContext) {
 }
 
 bool Unicode::triggerDirect(KeyEvent &keyEvent) {
-    if (!data_.load())
+    if (!data_.load()) {
         return false;
+    }
     auto *inputContext = keyEvent.inputContext();
     auto *state = inputContext->propertyFor(&factory_);
     state->mode_ = UnicodeMode::Direct;
@@ -158,7 +159,7 @@ bool Unicode::triggerDirect(KeyEvent &keyEvent) {
 }
 
 void Unicode::handleSearch(KeyEvent &keyEvent) {
-    auto inputContext = keyEvent.inputContext();
+    auto *inputContext = keyEvent.inputContext();
     auto *state = inputContext->propertyFor(&factory_);
     auto candidateList = inputContext->inputPanel().candidateList();
     if (candidateList) {
@@ -214,26 +215,27 @@ void Unicode::handleSearch(KeyEvent &keyEvent) {
                 UserInterfaceComponent::InputPanel);
             return;
         }
+
+        if (keyEvent.key().check(FcitxKey_Return) ||
+            keyEvent.key().check(FcitxKey_KP_Enter)) {
+            keyEvent.accept();
+            if (!candidateList->empty() && candidateList->cursorIndex() >= 0) {
+                candidateList->candidate(candidateList->cursorIndex())
+                    .select(inputContext);
+            }
+            return;
+        }
     }
 
     // and by pass all modifier
     if (keyEvent.key().isModifier() || keyEvent.key().hasModifier()) {
         return;
     }
+    keyEvent.accept();
     if (keyEvent.key().check(FcitxKey_Escape)) {
-        keyEvent.accept();
         state->reset(inputContext);
         return;
     }
-    if (keyEvent.key().check(FcitxKey_Return) ||
-        keyEvent.key().check(FcitxKey_KP_Enter)) {
-        keyEvent.accept();
-        if (candidateList->size() > 0 && candidateList->cursorIndex() >= 0) {
-            candidateList->candidate(candidateList->cursorIndex())
-                .select(inputContext);
-        }
-        return;
-    }
     if (keyEvent.key().check(FcitxKey_BackSpace)) {
         if (state->buffer_.empty()) {
             state->reset(inputContext);
@@ -246,7 +248,6 @@ void Unicode::handleSearch(KeyEvent &keyEvent) {
                 }
             }
         }
-        keyEvent.accept();
         return;
     }
 
@@ -256,7 +257,7 @@ void Unicode::handleSearch(KeyEvent &keyEvent) {
 
     // compose is invalid, ignore it.
     if (!compose) {
-        return keyEvent.accept();
+        return;
     }
 
     if (!compose->empty()) {
@@ -264,7 +265,6 @@ void Unicode::handleSearch(KeyEvent &keyEvent) {
     } else {
         state->buffer_.type(Key::keySymToUnicode(keyEvent.key().sym()));
     }
-    keyEvent.accept();
 
     updateUI(inputContext);
 }
@@ -287,7 +287,7 @@ bool bufferIsValid(const std::string &input, uint32_t *ret) {
 }
 
 void Unicode::handleDirect(KeyEvent &keyEvent) {
-    auto inputContext = keyEvent.inputContext();
+    auto *inputContext = keyEvent.inputContext();
     auto *state = inputContext->propertyFor(&factory_);
     keyEvent.accept();
     // by pass all modifier
@@ -361,19 +361,18 @@ void Unicode::updateUI(InputContext *inputContext, bool trigger) {
         } else {
             inputContext->inputPanel().setPreedit(preedit);
         }
-        FCITX_INFO() << preedit.toString();
     } else if (state->mode_ == UnicodeMode::Search) {
         if (!state->buffer_.empty()) {
             auto result = data_.find(state->buffer_.userInput());
             auto candidateList = std::make_unique<CommonCandidateList>();
             candidateList->setPageSize(
                 instance_->globalConfig().defaultPageSize());
-            for (auto c : result) {
-                if (utf8::UCS4IsValid(c)) {
-                    candidateList->append<UnicodeCandidateWord>(this, c);
+            for (auto chr : result) {
+                if (utf8::UCS4IsValid(chr)) {
+                    candidateList->append<UnicodeCandidateWord>(this, chr);
                 }
             }
-            if (candidateList->size()) {
+            if (!candidateList->empty()) {
                 candidateList->setGlobalCursorIndex(0);
             }
             candidateList->setSelectionKey(selectionKeys_);
@@ -416,18 +415,18 @@ void Unicode::updateUI(InputContext *inputContext, bool trigger) {
                 // Hard limit to prevent do too much lookup.
                 constexpr int limit = 20;
                 int counter = 0;
-                for (auto c : utf8::MakeUTF8CharRange(str)) {
-                    if (!seenChar.insert(c).second) {
+                for (auto chr : utf8::MakeUTF8CharRange(str)) {
+                    if (!seenChar.insert(chr).second) {
                         continue;
                     }
-                    auto name = data_.name(c);
+                    auto name = data_.name(chr);
                     std::string display;
                     if (!name.empty()) {
                         display = fmt::format("{0} U+{1:04X} {2}",
-                                              utf8::UCS4ToUTF8(c), c, name);
+                                              utf8::UCS4ToUTF8(chr), chr, name);
                     } else {
                         display = fmt::format("{0} U+{1:04X}",
-                                              utf8::UCS4ToUTF8(c), c);
+                                              utf8::UCS4ToUTF8(chr), chr);
                     }
                     candidateList->append<DisplayOnlyCandidateWord>(
                         Text(display));
@@ -440,7 +439,7 @@ void Unicode::updateUI(InputContext *inputContext, bool trigger) {
             candidateList->setSelectionKey(KeyList(10));
             candidateList->setPageSize(
                 instance_->globalConfig().defaultPageSize());
-            if (candidateList->size()) {
+            if (!candidateList->empty()) {
                 candidateList->setGlobalCursorIndex(0);
             }
 
