@@ -36,5 +36,7 @@ FCITX_ADDON_DECLARE_FUNCTION(WaylandModule, reloadXkbOption, void());
 FCITX_ADDON_DECLARE_FUNCTION(WaylandModule, openConnection,
                              bool(const std::string &));
 FCITX_ADDON_DECLARE_FUNCTION(WaylandModule, openConnectionSocket, bool(int fd));
+FCITX_ADDON_DECLARE_FUNCTION(WaylandModule, reopenConnectionSocket,
+                             bool(const std::string &name, int fd));
 
 #endif // _FCITX_MODULES_WAYLAND_WAYLAND_PUBLIC_H_
