@@ -7,18 +7,28 @@
 
 #include "waylandmodule.h"
 #include <fcntl.h>
+#include <algorithm>
+#include <cstdlib>
+#include <ctime>
+#include <memory>
 #include <stdexcept>
 #include <string>
 #include <gio/gio.h>
 #include <wayland-client.h>
 #include "fcitx-config/iniparser.h"
+#include "fcitx-utils/event.h"
 #include "fcitx-utils/log.h"
 #include "fcitx-utils/misc.h"
+#include "fcitx-utils/misc_p.h"
 #include "fcitx-utils/standardpath.h"
 #include "fcitx-utils/stringutils.h"
+#include "fcitx-utils/trackableobject.h"
+#include "fcitx/inputcontext.h"
 #include "fcitx/instance.h"
 #include "fcitx/misc_p.h"
 #include "config.h"
+#include "notifications_public.h"
+#include "waylandeventreader.h"
 #include "wl_seat.h"
 
 #ifdef ENABLE_DBUS
@@ -33,10 +43,6 @@ namespace fcitx {
 
 FCITX_DEFINE_LOG_CATEGORY(wayland_log, "wayland");
 
-#define FCITX_WAYLAND_INFO() FCITX_LOGC(::fcitx::wayland_log, Info)
-#define FCITX_WAYLAND_ERROR() FCITX_LOGC(::fcitx::wayland_log, Error)
-#define FCITX_WAYLAND_DEBUG() FCITX_LOGC(::fcitx::wayland_log, Debug)
-
 namespace {
 bool isKDE5() {
     static const DesktopType desktop = getDesktopType();
@@ -74,21 +80,34 @@ WaylandConnection::WaylandConnection(WaylandModule *wayland, std::string name)
     : parent_(wayland), name_(std::move(name)) {
     wl_display *display = nullptr;
     {
-        std::unique_ptr<ScopedEnvvar> env;
+        std::optional<ScopedEnvvar> env;
         if (wayland_log().checkLogLevel(Debug)) {
-            env = std::make_unique<ScopedEnvvar>("WAYLAND_DEBUG", "1");
+            env.emplace("WAYLAND_DEBUG", "1");
+        }
+        // When WAYLAND_SOCKET is set, wl_display_connect will use it
+        // regardlessly, no matter it is valid or not.
+        if (getenv("WAYLAND_SOCKET")) {
+            isWaylandSocket_ = true;
         }
         display = wl_display_connect(name_.empty() ? nullptr : name_.c_str());
     }
     if (!display) {
         throw std::runtime_error("Failed to open wayland connection");
     }
+    if (!isWaylandSocket_ && name_.empty()) {
+        // This is same as wl_display_connect(nullptr) logic.
+        realName_ = "wayland-0";
+        if (auto displayEnv = getenv("WAYLAND_DISPLAY")) {
+            realName_ = displayEnv;
+        }
+    }
     init(display);
 }
 
 WaylandConnection::WaylandConnection(WaylandModule *wayland, std::string name,
-                                     int fd)
-    : parent_(wayland), name_(std::move(name)) {
+                                     int fd, std::string realName)
+    : parent_(wayland), name_(std::move(name)), realName_(std::move(realName)),
+      isWaylandSocket_(true) {
     wl_display *display = nullptr;
     {
         std::unique_ptr<ScopedEnvvar> env;
@@ -106,17 +125,8 @@ WaylandConnection::WaylandConnection(WaylandModule *wayland, std::string name,
 WaylandConnection::~WaylandConnection() {}
 
 void WaylandConnection::init(wl_display *display) {
-
     display_ = std::make_unique<wayland::Display>(display);
 
-    auto &eventLoop = parent_->instance()->eventLoop();
-    ioEvent_ =
-        eventLoop.addIOEvent(display_->fd(), IOEventFlag::In,
-                             [this](EventSource *, int, IOEventFlags flags) {
-                                 onIOEvent(flags);
-                                 return true;
-                             });
-
     group_ = std::make_unique<FocusGroup>(
         "wayland:" + name_, parent_->instance()->inputContextManager());
 
@@ -132,9 +142,10 @@ void WaylandConnection::init(wl_display *display) {
                 keyboards_.erase(static_cast<wayland::WlSeat *>(ptr.get()));
             }
         });
-    for (auto seat : display_->getGlobals<wayland::WlSeat>()) {
+    for (const auto &seat : display_->getGlobals<wayland::WlSeat>()) {
         setupKeyboard(seat.get());
     }
+    eventReader_ = std::make_unique<WaylandEventReader>(this);
 }
 
 void WaylandConnection::finish() { parent_->removeConnection(name_); }
@@ -147,32 +158,6 @@ void WaylandConnection::setupKeyboard(wayland::WlSeat *seat) {
     });
 }
 
-void WaylandConnection::onIOEvent(IOEventFlags flags) {
-    if ((flags & IOEventFlag::Err) || (flags & IOEventFlag::Hup)) {
-        FCITX_WAYLAND_ERROR() << "Received error on socket.";
-        return finish();
-    }
-
-    if (wl_display_prepare_read(*display_) == 0) {
-        if (flags & IOEventFlag::In) {
-            wl_display_read_events(*display_);
-        } else {
-            wl_display_cancel_read(*display_);
-        }
-    }
-
-    if (wl_display_dispatch(*display_) < 0) {
-        error_ = wl_display_get_error(*display_);
-        FCITX_LOGC_IF(wayland_log, Error, error_ != 0)
-            << "Wayland connection got error: " << error_;
-        if (error_ != 0) {
-            return finish();
-        }
-    }
-
-    wl_display_flush(*display_);
-}
-
 WaylandModule::WaylandModule(fcitx::Instance *instance)
     : instance_(instance), isWaylandSession_(isSessionType("wayland")) {
 
@@ -185,7 +170,11 @@ WaylandModule::WaylandModule(fcitx::Instance *instance)
     delayedReloadXkbOption_->setEnabled(false);
 
     reloadConfig();
-    openConnection("");
+    if (!containerContains(instance->addonManager().addonOptions("wayland"),
+                           "nodefault")) {
+        openConnection("");
+    }
+
     reloadXkbOption();
 
 #ifdef ENABLE_DBUS
@@ -207,6 +196,13 @@ WaylandModule::WaylandModule(fcitx::Instance *instance)
                 setLayoutToGNOME();
             }
         }));
+
+    deferredDiagnose_ = instance_->eventLoop().addTimeEvent(
+        CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 7000000, 0,
+        [this](EventSourceTime *, uint64_t) {
+            selfDiagnose();
+            return true;
+        });
 #endif
 }
 
@@ -217,52 +213,140 @@ bool WaylandModule::openConnection(const std::string &name) {
         return false;
     }
 
-    WaylandConnection *newConnection = nullptr;
-    try {
-        auto iter = conns_.emplace(std::piecewise_construct,
-                                   std::forward_as_tuple(name),
-                                   std::forward_as_tuple(this, name));
-        newConnection = &iter.first->second;
-    } catch (const std::exception &e) {
-        FCITX_ERROR() << e.what();
-    }
-    if (newConnection) {
-        onConnectionCreated(*newConnection);
-        return true;
-    }
-    return false;
-}
-
-bool WaylandModule::openConnectionSocket(int fd) {
-    UnixFD guard = UnixFD::own(fd);
-    auto name = stringutils::concat("socket:", fd);
-
-    if (conns_.count(name)) {
-        return false;
-    }
-
-    for (const auto &[name, connection] : conns_) {
-        if (connection.display()->fd() == fd) {
+    if (auto defaultConnection = findValue(conns_, "")) {
+        if (name == defaultConnection->get()->realName()) {
             return false;
         }
     }
 
     WaylandConnection *newConnection = nullptr;
     try {
-        auto iter = conns_.emplace(std::piecewise_construct,
-                                   std::forward_as_tuple(name),
-                                   std::forward_as_tuple(this, name, fd));
+        auto connection = std::make_unique<WaylandConnection>(this, name);
+        auto iter = conns_.emplace(
+            std::piecewise_construct, std::forward_as_tuple(name),
+            std::forward_as_tuple(std::move(connection)));
+        newConnection = iter.first->second.get();
+    } catch (const std::exception &e) {
+        FCITX_ERROR() << e.what();
+    }
+    if (newConnection) {
+        refreshCanRestart();
+        onConnectionCreated(*newConnection);
+        return true;
+    }
+
+    return false;
+}
+
+bool WaylandModule::openConnectionSocket(int fd) {
+    auto name = stringutils::concat("socket:", fd);
+    return openConnectionSocketWithName(fd, name, "");
+}
+
+bool WaylandModule::openConnectionSocketWithName(int fd,
+                                                 const std::string &name,
+                                                 const std::string &realName) {
+    UnixFD guard = UnixFD::own(fd);
+
+    if (conns_.count(name)) {
+        return false;
+    }
+
+    for (const auto &[name, connection] : conns_) {
+        if (connection->display()->fd() == fd) {
+            return false;
+        }
+    }
+
+    WaylandConnection *newConnection = nullptr;
+    try {
+        auto connection =
+            std::make_unique<WaylandConnection>(this, name, fd, realName);
+        auto iter = conns_.emplace(
+            std::piecewise_construct, std::forward_as_tuple(name),
+            std::forward_as_tuple(std::move(connection)));
         guard.release();
-        newConnection = &iter.first->second;
+        newConnection = iter.first->second.get();
     } catch (const std::exception &e) {
     }
     if (newConnection) {
+        refreshCanRestart();
         onConnectionCreated(*newConnection);
         return true;
     }
     return false;
 }
 
+bool WaylandModule::reopenConnectionSocket(const std::string &displayName,
+                                           int fd) {
+    UnixFD guard = UnixFD::own(fd);
+    std::string name = displayName;
+
+    auto iter = conns_.find(name);
+    if (iter == conns_.end() && !name.empty()) {
+        do {
+            // If it's in flatpak, WAYLAND_DISPLAY may be overrrided, so don't
+            // compare it with default connection.
+            if (!isInFlatpak()) {
+                iter = conns_.find("");
+                if (iter != conns_.end() && iter->second->realName() == name) {
+                    name = "";
+                    break;
+                }
+
+                if (auto displayEnv = getenv("WAYLAND_DISPLAY")) {
+                    if (name == displayEnv) {
+                        name = "";
+                    }
+                }
+            }
+            return openConnectionSocketWithName(guard.release(), name,
+                                                displayName);
+
+        } while (0);
+    }
+
+    for (const auto &[name, connection] : conns_) {
+        if (connection->display()->fd() == fd) {
+            return false;
+        }
+    }
+
+    // Record all the IC in the old connection.
+    std::vector<TrackableObjectReference<InputContext>> ics;
+    iter->second->focusGroup()->foreach([&ics](InputContext *ic) {
+        ics.push_back(ic->watch());
+        return true;
+    });
+
+    std::unique_ptr<WaylandConnection> newConnection;
+    try {
+        newConnection =
+            std::make_unique<WaylandConnection>(this, name, fd, displayName);
+        guard.release();
+    } catch (const std::exception &e) {
+    }
+    if (newConnection) {
+        // At this point connection is already constructed, now we try to
+        // replace it.
+        onConnectionClosed(*iter->second);
+        iter->second = std::move(newConnection);
+        refreshCanRestart();
+        onConnectionCreated(*iter->second);
+        // Transfer it to new connection's IC group, we do create two focus
+        // group with same name, but well, ic manager doesn't check that.
+        for (auto icRef : ics) {
+            if (auto ic = icRef.get()) {
+                if (!ic->focusGroup()) {
+                    ic->setFocusGroup(iter->second->focusGroup());
+                }
+            }
+        }
+        return true;
+    }
+    return false;
+}
+
 void WaylandModule::removeConnection(const std::string &name) {
     FCITX_WAYLAND_INFO() << "Connection removed " << name;
     if (name.empty() && instance_->exitWhenMainDisplayDisconnected() &&
@@ -271,8 +355,9 @@ void WaylandModule::removeConnection(const std::string &name) {
     }
     auto iter = conns_.find(name);
     if (iter != conns_.end()) {
-        onConnectionClosed(iter->second);
+        onConnectionClosed(*iter->second);
         conns_.erase(iter);
+        refreshCanRestart();
     }
 }
 
@@ -282,7 +367,8 @@ WaylandModule::addConnectionCreatedCallback(WaylandConnectionCreated callback) {
 
     for (auto &p : conns_) {
         auto &conn = p.second;
-        (**result->handler())(conn.name(), *conn.display(), conn.focusGroup());
+        (**result->handler())(conn->name(), *conn->display(),
+                              conn->focusGroup());
     }
     return result;
 }
@@ -303,6 +389,14 @@ void WaylandModule::onConnectionClosed(WaylandConnection &conn) {
         callback(conn.name(), *conn.display());
     }
 }
+
+void WaylandModule::refreshCanRestart() {
+    setCanRestart(std::all_of(conns_.begin(), conns_.end(),
+                              [](const decltype(conns_)::value_type &conn) {
+                                  return !conn.second->isWaylandSocket();
+                              }));
+}
+
 void WaylandModule::reloadXkbOption() {
     delayedReloadXkbOption_->setNextInterval(30000);
     delayedReloadXkbOption_->setOneShot();
@@ -332,7 +426,7 @@ void WaylandModule::reloadXkbOptionReal() {
         auto model = config.valueByPath("Layout/Model");
         auto options = config.valueByPath("Layout/Options");
         xkbOption = (options ? *options : "");
-        instance_->setXkbParameters(connection->focusGroup()->display(),
+        instance_->setXkbParameters((*connection)->focusGroup()->display(),
                                     DEFAULT_XKB_RULES, model ? *model : "",
                                     *xkbOption);
         FCITX_WAYLAND_DEBUG()
@@ -347,8 +441,9 @@ void WaylandModule::reloadXkbOptionReal() {
             if (value) {
                 auto options = g_strjoinv(",", value);
                 xkbOption = (options ? options : "");
-                instance_->setXkbParameters(connection->focusGroup()->display(),
-                                            DEFAULT_XKB_RULES, "", *xkbOption);
+                instance_->setXkbParameters(
+                    (*connection)->focusGroup()->display(), DEFAULT_XKB_RULES,
+                    "", *xkbOption);
                 FCITX_WAYLAND_DEBUG() << "GNOME xkb options=" << *xkbOption;
                 g_free(options);
                 g_strfreev(value);
@@ -410,7 +505,6 @@ void WaylandModule::setLayoutToKDE5() {
 
 void WaylandModule::setLayoutToGNOME() {
 #ifdef ENABLE_DBUS
-
     auto layoutAndVariant = parseLayout(
         instance_->inputMethodManager().currentGroup().defaultLayout());
 
@@ -439,6 +533,140 @@ void WaylandModule::setLayoutToGNOME() {
 #endif
 }
 
+void WaylandModule::selfDiagnose() {
+    if (!isWaylandSession_) {
+        return;
+    }
+
+    auto *connection = findValue(conns_, "");
+    if (!connection) {
+        return;
+    }
+
+    if (!notifications()) {
+        return;
+    }
+
+    bool isWaylandIM = false;
+    if (isInFlatpak()) {
+        // In flatpak, ReopenWaylandConnection will not replace existing connection.
+        for (const auto &[_, conn] : conns_) {
+            conn->focusGroup()->foreach([&isWaylandIM](InputContext *ic) {
+                if (stringutils::startsWith(ic->frontendName(), "wayland")) {
+                    isWaylandIM = true;
+                    return false;
+                }
+                return true;
+            });
+            if (isWaylandIM) {
+                break;
+            }
+        }
+    } else {
+        (*connection)->focusGroup()->foreach([&isWaylandIM](InputContext *ic) {
+            if (stringutils::startsWith(ic->frontendName(), "wayland")) {
+                isWaylandIM = true;
+                return false;
+            }
+            return true;
+        });
+    }
+
+    auto sendMessage = [this](const std::string &category,
+                              const std::string &message) {
+        notifications()->call<INotifications::showTip>(
+            category, _("Fcitx"), "fcitx", _("Wayland Diagnose"), message,
+            60000);
+    };
+
+    auto *gtk = getenv("GTK_IM_MODULE");
+    auto *qt = getenv("QT_IM_MODULE");
+    std::string gtkIM = gtk ? gtk : "";
+    std::string qtIM = qt ? qt : "";
+
+    std::vector<std::string> messages;
+    const auto desktop = getDesktopType();
+    if (desktop == DesktopType::KDE5) {
+        if (!isWaylandIM) {
+            sendMessage(
+                "wayland-diagnose-kde",
+                _("Fcitx should be launched by KWin under KDE Wayland in order "
+                  "to use Wayland input method frontend. This can improve the "
+                  "experience when using Fcitx on Wayland. To "
+                  "configure this, you need to go to \"System Settings\" -> "
+                  "\"Virtual "
+                  "keyboard\" and select \"Fcitx 5\" from it. You may also "
+                  "need to disable tools that launches input method, such as "
+                  "imsettings on Fedora, or im-config on Debian/Ubuntu. For "
+                  "more details see "
+                  "https://fcitx-im.org/wiki/"
+                  "Using_Fcitx_5_on_Wayland#KDE_Plasma"));
+        } else if (!gtkIM.empty() || !qtIM.empty()) {
+            sendMessage("wayland-diagnose-kde",
+                        _("Detect GTK_IM_MODULE and QT_IM_MODULE being set and "
+                          "Wayland Input method frontend is working. It is "
+                          "recommended to unset GTK_IM_MODULE and QT_IM_MODULE "
+                          "and use Wayland input method frontend instead. For "
+                          "more details see "
+                          "https://fcitx-im.org/wiki/"
+                          "Using_Fcitx_5_on_Wayland#KDE_Plasma"));
+        }
+    } else if (desktop == DesktopType::GNOME) {
+        if (instance_->currentUI() != "kimpanel") {
+            sendMessage(
+                "wayland-diagnose-gnome",
+                _("It is recommended to install Input Method Panel GNOME "
+                  "Shell Extensions to provide the input method popup. "
+                  "https://extensions.gnome.org/extension/261/kimpanel/ "
+                  "Otherwise you may not be able to see input method popup "
+                  "when typing in GNOME Shell's activities search box. For "
+                  "more details "
+                  "see "
+                  "https://fcitx-im.org/wiki/Using_Fcitx_5_on_Wayland#GNOME"));
+        }
+    } else {
+        // It is not clear whether compositor is supported, only warn if wayland
+        // im is being used..
+        if (isWaylandIM) {
+            // Sway's input popup is not merged.
+            if (!gtkIM.empty() && gtkIM != "wayland" &&
+                desktop != DesktopType::Sway) {
+                messages.push_back(
+                    _("Detect GTK_IM_MODULE being set and "
+                      "Wayland Input method frontend is working. It is "
+                      "recommended to unset GTK_IM_MODULE "
+                      "and use Wayland input method frontend instead."));
+            }
+        }
+
+        std::unordered_set<std::string> groupLayouts;
+        for (const auto &groupName : instance_->inputMethodManager().groups()) {
+            if (const auto *group =
+                    instance_->inputMethodManager().group(groupName)) {
+                groupLayouts.insert(group->defaultLayout());
+            }
+            if (groupLayouts.size() >= 2) {
+                messages.push_back(
+                    _("Sending keyboard layout configuration to wayland "
+                      "compositor from Fcitx is "
+                      "not yet supported on current desktop. You may still use "
+                      "Fcitx's internal layout conversion by adding layout as "
+                      "input method to the input method group."));
+                break;
+            }
+        }
+
+        if (!messages.empty()) {
+            messages.push_back(
+                _("For more details see "
+                  "https://fcitx-im.org/wiki/Using_Fcitx_5_on_Wayland"));
+
+            sendMessage("wayland-diagnose-other",
+                        stringutils::join(messages, "\n"));
+        }
+    }
+}
+
 class WaylandModuleFactory : public AddonFactory {
 public:
     AddonInstance *create(AddonManager *manager) override {
