@@ -8,9 +8,12 @@
 #define _FCITX_MODULES_WAYLAND_WAYLANDMODULE_H_
 
 #include <cstdint>
+#include <memory>
+#include <string>
 #include "fcitx-config/iniparser.h"
 #include "fcitx-utils/event.h"
 #include "fcitx-utils/i18n.h"
+#include "fcitx-utils/log.h"
 #include "fcitx/addonfactory.h"
 #include "fcitx/addoninstance.h"
 #include "fcitx/addonmanager.h"
@@ -24,6 +27,7 @@
 namespace fcitx {
 
 class WaylandModule;
+class WaylandEventReader;
 
 FCITX_CONFIGURATION(
     WaylandConfig,
@@ -61,29 +65,37 @@ private:
 class WaylandConnection {
 public:
     WaylandConnection(WaylandModule *wayland, std::string name);
-    WaylandConnection(WaylandModule *wayland, std::string name, int fd);
+    WaylandConnection(WaylandModule *wayland, std::string name, int fd,
+                      std::string realName);
     ~WaylandConnection();
 
     const std::string &name() const { return name_; }
+    const std::string &realName() const {
+        return name_.empty() ? realName_ : name_;
+    }
     wayland::Display *display() const { return display_.get(); }
     FocusGroup *focusGroup() const { return group_.get(); }
+    auto *parent() const { return parent_; }
+
+    bool isWaylandSocket() const { return isWaylandSocket_; }
 
 private:
     void init(wl_display *display);
-    void onIOEvent(IOEventFlags flags);
     void finish();
     void setupKeyboard(wayland::WlSeat *seat);
 
     WaylandModule *parent_;
     std::string name_;
+    std::string realName_;
     // order matters, callback in ioEvent_ uses display_.
     std::unique_ptr<wayland::Display> display_;
-    std::unique_ptr<EventSourceIO> ioEvent_;
+    std::unique_ptr<WaylandEventReader> eventReader_;
     std::unique_ptr<FocusGroup> group_;
     int error_ = 0;
     ScopedConnection panelConn_, panelRemovedConn_;
     std::unordered_map<wayland::WlSeat *, std::unique_ptr<WaylandKeyboard>>
         keyboards_;
+    bool isWaylandSocket_ = false;
 };
 
 class WaylandModule : public AddonInstance {
@@ -93,6 +105,9 @@ public:
 
     bool openConnection(const std::string &name);
     bool openConnectionSocket(int fd);
+    bool openConnectionSocketWithName(int fd, const std::string &name,
+                                      const std::string &realName);
+    bool reopenConnectionSocket(const std::string &name, int fd);
     void removeConnection(const std::string &name);
 
     std::unique_ptr<HandlerTableEntry<WaylandConnectionCreated>>
@@ -100,7 +115,6 @@ public:
     std::unique_ptr<HandlerTableEntry<WaylandConnectionClosed>>
     addConnectionClosedCallback(WaylandConnectionClosed callback);
     void reloadXkbOption();
-    wl_registry *getRegistry(const std::string &name);
 
     const Configuration *getConfig() const override { return &config_; }
     void setConfig(const RawConfig &config) override {
@@ -109,20 +123,24 @@ public:
     }
     void reloadConfig() override;
 
+    void selfDiagnose();
+
 private:
     void onConnectionCreated(WaylandConnection &conn);
     void onConnectionClosed(WaylandConnection &conn);
+    void refreshCanRestart();
     void reloadXkbOptionReal();
     void setLayoutToGNOME();
     void setLayoutToKDE5();
 
     FCITX_ADDON_DEPENDENCY_LOADER(dbus, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(xcb, instance_->addonManager());
+    FCITX_ADDON_DEPENDENCY_LOADER(notifications, instance_->addonManager());
 
     Instance *instance_;
     WaylandConfig config_;
     bool isWaylandSession_ = false;
-    std::unordered_map<std::string, WaylandConnection> conns_;
+    std::unordered_map<std::string, std::unique_ptr<WaylandConnection>> conns_;
     HandlerTable<WaylandConnectionCreated> createdCallbacks_;
     HandlerTable<WaylandConnectionClosed> closedCallbacks_;
     FCITX_ADDON_EXPORT_FUNCTION(WaylandModule, addConnectionCreatedCallback);
@@ -130,11 +148,20 @@ private:
     FCITX_ADDON_EXPORT_FUNCTION(WaylandModule, reloadXkbOption);
     FCITX_ADDON_EXPORT_FUNCTION(WaylandModule, openConnection);
     FCITX_ADDON_EXPORT_FUNCTION(WaylandModule, openConnectionSocket);
+    FCITX_ADDON_EXPORT_FUNCTION(WaylandModule, reopenConnectionSocket);
 
     std::vector<std::unique_ptr<HandlerTableEntry<EventHandler>>>
         eventHandlers_;
     std::unique_ptr<EventSourceTime> delayedReloadXkbOption_;
+    std::unique_ptr<EventSourceTime> deferredDiagnose_;
 };
+
+FCITX_DECLARE_LOG_CATEGORY(wayland_log);
+
+#define FCITX_WAYLAND_INFO() FCITX_LOGC(::fcitx::wayland_log, Info)
+#define FCITX_WAYLAND_ERROR() FCITX_LOGC(::fcitx::wayland_log, Error)
+#define FCITX_WAYLAND_DEBUG() FCITX_LOGC(::fcitx::wayland_log, Debug)
+
 } // namespace fcitx
 
 #endif // _FCITX_MODULES_WAYLAND_WAYLANDMODULE_H_
