@@ -176,7 +176,6 @@ void XCBKeyboard::updateKeymap() {
     if (!context_) {
         return;
     }
-    xcb_flush(connection());
     initDefaultLayout();
 
     keymap_.reset(nullptr);
@@ -514,7 +513,6 @@ bool XCBKeyboard::setLayoutByName(const std::string &layout,
 #endif
     xcb_xkb_latch_lock_state(connection(), XCB_XKB_ID_USE_CORE_KBD, 0, 0, true,
                              index, 0, false, 0);
-    xcb_flush(connection());
     return true;
 }
 
@@ -541,6 +539,11 @@ bool XCBKeyboard::handleEvent(xcb_generic_event_t *event) {
         switch (xkbEvent->any.xkbType) {
         case XCB_XKB_STATE_NOTIFY: {
             xcb_xkb_state_notify_event_t *state = &xkbEvent->state_notify;
+            FCITX_XCB_DEBUG()
+                << "XCB_XKB_STATE_NOTIFY depressed:"
+                << static_cast<unsigned int>(state->baseMods)
+                << " latched:" << static_cast<unsigned int>(state->latchedMods)
+                << " locked:" << static_cast<unsigned int>(state->lockedMods);
             xkb_state_update_mask(state_.get(), state->baseMods,
                                   state->latchedMods, state->lockedMods,
                                   state->baseGroup, state->latchedGroup,
