@@ -39,6 +39,7 @@ public:
     XCBModule(Instance *instance);
 
     void openConnection(const std::string &name);
+    bool openConnectionChecked(const std::string &name);
     void removeConnection(const std::string &name);
     std::string mainDisplay() { return mainDisplay_; }
     const XCBConfig &config() const { return config_; }
@@ -74,7 +75,10 @@ public:
 
     void setXkbOption(const std::string &name, const std::string &option);
 
+    bool exists(const std::string &name);
+
     FCITX_ADDON_DEPENDENCY_LOADER(notifications, instance_->addonManager());
+    FCITX_ADDON_DEPENDENCY_LOADER(waylandim, instance_->addonManager());
 
 private:
     void onConnectionCreated(XCBConnection &conn);
@@ -87,6 +91,7 @@ private:
     HandlerTable<XCBConnectionClosed> closedCallbacks_;
     std::string mainDisplay_;
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, openConnection);
+    FCITX_ADDON_EXPORT_FUNCTION(XCBModule, openConnectionChecked);
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, addEventFilter);
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, addConnectionCreatedCallback);
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, addConnectionClosedCallback);
@@ -99,6 +104,7 @@ private:
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, mainDisplay);
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, setXkbOption);
     FCITX_ADDON_EXPORT_FUNCTION(XCBModule, isXWayland);
+    FCITX_ADDON_EXPORT_FUNCTION(XCBModule, exists)
 };
 
 FCITX_DECLARE_LOG_CATEGORY(xcb_log);
