@@ -7,6 +7,7 @@
 
 #include "classicui.h"
 #include <fcntl.h>
+#include <optional>
 #include <string>
 #include <string_view>
 #include "fcitx-config/iniparser.h"
@@ -14,6 +15,7 @@
 #include "fcitx-utils/misc_p.h"
 #include "fcitx-utils/standardpath.h"
 #include "fcitx-utils/utf8.h"
+#include "fcitx/event.h"
 #include "fcitx/inputcontext.h"
 #include "fcitx/inputcontextmanager.h"
 #include "fcitx/instance.h"
@@ -30,6 +32,7 @@
 #ifdef ENABLE_DBUS
 #include "fcitx-utils/dbus/variant.h"
 #include "dbus_public.h"
+#include "portalsettingmonitor.h"
 #endif
 
 namespace fcitx::classicui {
@@ -38,23 +41,19 @@ FCITX_DEFINE_LOG_CATEGORY(classicui_logcategory, "classicui");
 
 using AccentColorDBusType = FCITX_STRING_TO_DBUS_TYPE("(ddd)");
 
-namespace {
-constexpr const char XDG_PORTAL_DESKTOP_SERVICE[] =
-    "org.freedesktop.portal.Desktop";
-constexpr const char XDG_PORTAL_DESKTOP_PATH[] =
-    "/org/freedesktop/portal/desktop";
-constexpr const char XDG_PORTAL_DESKTOP_SETTINGS_INTERFACE[] =
-    "org.freedesktop.portal.Settings";
-} // namespace
-
 ClassicUI::ClassicUI(Instance *instance) : instance_(instance) {
-    reloadConfig();
 
 #ifdef ENABLE_DBUS
-    dbus::VariantTypeRegistry::defaultRegistry()
-        .registerType<AccentColorDBusType>();
+    if (auto dbusAddon = dbus()) {
+        dbus::VariantTypeRegistry::defaultRegistry()
+            .registerType<AccentColorDBusType>();
+        settingMonitor_ = std::make_unique<PortalSettingMonitor>(
+            *dbusAddon->call<IDBusModule::bus>());
+    }
 #endif
 
+    reloadConfig();
+
 #ifdef ENABLE_X11
     if (auto *xcbAddon = xcb()) {
         xcbCreatedCallback_ =
@@ -103,6 +102,27 @@ ClassicUI::ClassicUI(Instance *instance) : instance_(instance) {
             return true;
         });
     deferedReloadTheme_->setEnabled(false);
+
+    // Since kimpanel may call classicui
+    persistentEventHandlers_.emplace_back(instance_->watchEvent(
+        EventType::FocusGroupFocusChanged, EventWatcherPhase::Default,
+        [this](Event &event) {
+            auto &focusEvent =
+                static_cast<FocusGroupFocusChangedEvent &>(event);
+            if (!focusEvent.newFocus()) {
+                if (auto ui = uiForDisplay(focusEvent.group()->display())) {
+                    ui->update(UserInterfaceComponent::InputPanel, nullptr);
+                }
+            }
+        }));
+    persistentEventHandlers_.emplace_back(instance_->watchEvent(
+        EventType::UIChanged, EventWatcherPhase::Default, [this](Event &) {
+            if (instance_->currentUI() == "kimpanel") {
+                deferedReloadTheme_->setOneShot();
+            } else if (instance_->currentUI() == "classicui") {
+                deferedReloadTheme_->setOneShot();
+            }
+        }));
 }
 
 ClassicUI::~ClassicUI() {}
@@ -114,7 +134,7 @@ void ClassicUI::reloadConfig() {
 
 void ClassicUI::reloadTheme() {
 #ifdef ENABLE_DBUS
-    auto parseMessage = [this](dbus::Variant &variant) {
+    auto parseMessage = [this](const dbus::Variant &variant) {
         if (variant.signature() == "u") {
             auto color = variant.dataAs<uint32_t>();
             auto oldIsDark = isDark_;
@@ -129,60 +149,19 @@ void ClassicUI::reloadTheme() {
     };
 
     if (dbus()) {
-        auto bus = dbus()->call<IDBusModule::bus>();
         if (*config_.useDarkTheme) {
-            if (!appearanceChangedSlot_) {
-                appearanceChangedSlot_ = bus->addMatch(
-                    dbus::MatchRule(
-                        XDG_PORTAL_DESKTOP_SERVICE, XDG_PORTAL_DESKTOP_PATH,
-                        XDG_PORTAL_DESKTOP_SETTINGS_INTERFACE, "SettingChanged",
-                        {"org.freedesktop.appearance", "color-scheme"}),
-                    [parseMessage](dbus::Message &msg) {
-                        if (msg.type() == dbus::MessageType::Signal &&
-                            msg.signature() == "ssv") {
-                            std::string interface, name;
-                            msg >> interface >> name;
-                            if (interface == "org.freedesktop.appearance" &&
-                                name == "color-scheme") {
-                                dbus::Variant variant;
-                                msg >> variant;
-                                parseMessage(variant);
-                            }
-                        }
-                        return true;
-                    });
-                auto call = bus->createMethodCall(
-                    "org.freedesktop.portal.Desktop",
-                    "/org/freedesktop/portal/desktop",
-                    "org.freedesktop.portal.Settings", "Read");
-                call << "org.freedesktop.appearance"
-                     << "color-scheme";
-                initialReadSlot_ =
-                    call.callAsync(1000000, [parseMessage](dbus::Message &msg) {
-                        // XDG portal seems didn't unwrap the variant.
-                        // Check this special case just in case.
-                        if (msg.isError()) {
-                            return true;
-                        }
-                        if (msg.signature() != "v") {
-                            return true;
-                        }
-                        dbus::Variant variant;
-                        msg >> variant;
-                        if (variant.signature() == "v") {
-                            variant = variant.dataAs<dbus::Variant>();
-                        }
-                        parseMessage(variant);
-                        return true;
-                    });
+            if (!darkModeEntry_ && settingMonitor_) {
+                darkModeEntry_ = settingMonitor_->watch(
+                    "org.freedesktop.appearance", "color-scheme", parseMessage);
             }
         } else {
-            appearanceChangedSlot_.reset();
+            darkModeEntry_.reset();
         }
     }
 #endif
 
     bool hasPlasmaTheme =
+        instance_->currentUI() == "kimpanel" ||
         (*config_.theme == PlasmaThemeName) ||
         (*config_.useDarkTheme && *config_.themeDark == PlasmaThemeName);
 
@@ -205,10 +184,16 @@ void ClassicUI::reloadTheme() {
                                      ? *config_.themeDark
                                      : *config_.theme;
 
+    if (instance_->currentUI() == "kimpanel") {
+        if (plasmaThemeWatchdog_) {
+            themeName = "plasma";
+        }
+    }
+
     theme_.load(themeName);
 
 #ifdef ENABLE_DBUS
-    auto parseAccentColor = [this](dbus::Variant &variant) {
+    auto parseAccentColor = [this](const dbus::Variant &variant) {
         if (variant.signature() == "(ddd)") {
             auto dbuscolor = variant.dataAs<AccentColorDBusType>();
             Color color;
@@ -227,58 +212,19 @@ void ClassicUI::reloadTheme() {
     };
 
     if (dbus()) {
-        auto bus = dbus()->call<IDBusModule::bus>();
         if (*config_.useAccentColor) {
-            if (!accentColorSlot_) {
-                accentColorSlot_ = bus->addMatch(
-                    dbus::MatchRule(
-                        XDG_PORTAL_DESKTOP_SERVICE, XDG_PORTAL_DESKTOP_PATH,
-                        XDG_PORTAL_DESKTOP_SETTINGS_INTERFACE, "SettingChanged",
-                        {"org.freedesktop.appearance", "accent-color"}),
-                    [parseAccentColor](dbus::Message &msg) {
-                        if (msg.type() == dbus::MessageType::Signal &&
-                            msg.signature() == "ssv") {
-                            std::string interface, name;
-                            msg >> interface >> name;
-                            if (interface == "org.freedesktop.appearance" &&
-                                name == "accent-color") {
-                                dbus::Variant variant;
-                                msg >> variant;
-                                parseAccentColor(variant);
-                            }
-                        }
-                        return true;
-                    });
-                auto call = bus->createMethodCall(
-                    "org.freedesktop.portal.Desktop",
-                    "/org/freedesktop/portal/desktop",
-                    "org.freedesktop.portal.Settings", "Read");
-                call << "org.freedesktop.appearance"
-                     << "accent-color";
-                accentColorReadSlot_ = call.callAsync(
-                    1000000, [parseAccentColor](dbus::Message &msg) {
-                        // XDG portal seems didn't unwrap the variant.
-                        // Check this special case just in case.
-                        if (msg.isError()) {
-                            return true;
-                        }
-                        if (msg.signature() != "v") {
-                            return true;
-                        }
-                        dbus::Variant variant;
-                        msg >> variant;
-                        if (variant.signature() == "v") {
-                            variant = variant.dataAs<dbus::Variant>();
-                        }
-                        parseAccentColor(variant);
-                        return true;
-                    });
+            if (!accentColorEntry_ && settingMonitor_) {
+                accentColorEntry_ =
+                    settingMonitor_->watch("org.freedesktop.appearance",
+                                           "accent-color", parseAccentColor);
             }
+        } else {
+            accentColorEntry_.reset();
+            accentColor_ = std::nullopt;
         }
-    } else {
-        accentColorSlot_.reset();
     }
 #endif
+
     theme_.populateColor(accentColor_);
 }
 
@@ -467,7 +413,7 @@ void ClassicUI::update(UserInterfaceComponent component,
                        InputContext *inputContext) {
     UIInterface *ui = nullptr;
     if (stringutils::startsWith(inputContext->display(), "wayland:") &&
-        !stringutils::startsWith(inputContext->frontend(), "wayland")) {
+        !stringutils::startsWith(inputContext->frontendName(), "wayland")) {
         // If display is wayland, but frontend is not, then we can only do X11
         // for now, though position is wrong. We don't know which is xwayland
         // unfortunately, hopefully main display is X wayland.
@@ -489,7 +435,7 @@ void ClassicUI::update(UserInterfaceComponent component,
     }
     CLASSICUI_DEBUG() << "Update component: " << static_cast<int>(component)
                       << " for IC program:" << inputContext->program()
-                      << " frontend:" << inputContext->frontend()
+                      << " frontend:" << inputContext->frontendName()
                       << " display:" << inputContext->display()
                       << " ui:" << (ui ? ui->name() : "(not available)");
 
