@@ -24,6 +24,7 @@
 #include "fcitx/userinterface.h"
 #include "classicui_public.h"
 #include "plasmathemewatchdog.h"
+#include "portalsettingmonitor.h"
 #include "theme.h"
 #ifdef ENABLE_X11
 #include "xcb_public.h"
@@ -200,6 +201,7 @@ public:
     FCITX_ADDON_DEPENDENCY_LOADER(xcb, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(wayland, instance_->addonManager());
     FCITX_ADDON_DEPENDENCY_LOADER(waylandim, instance_->addonManager());
+    FCITX_ADDON_DEPENDENCY_LOADER(dbus, instance_->addonManager());
     Instance *instance() const { return instance_; }
     const Configuration *getConfig() const override;
     void setConfig(const RawConfig &config) override {
@@ -227,7 +229,6 @@ public:
 
 private:
     FCITX_ADDON_DEPENDENCY_LOADER(notificationitem, instance_->addonManager());
-    FCITX_ADDON_DEPENDENCY_LOADER(dbus, instance_->addonManager());
     FCITX_ADDON_EXPORT_FUNCTION(ClassicUI, labelIcon);
     FCITX_ADDON_EXPORT_FUNCTION(ClassicUI, preferTextIcon);
     FCITX_ADDON_EXPORT_FUNCTION(ClassicUI, showLayoutNameInIcon);
@@ -252,15 +253,15 @@ private:
 
     std::unique_ptr<EventSource> deferedReloadTheme_;
 #ifdef ENABLE_DBUS
-    std::unique_ptr<dbus::Slot> appearanceChangedSlot_;
-    std::unique_ptr<dbus::Slot> initialReadSlot_;
-
-    std::unique_ptr<dbus::Slot> accentColorSlot_;
-    std::unique_ptr<dbus::Slot> accentColorReadSlot_;
+    std::unique_ptr<PortalSettingMonitor> settingMonitor_;
+    std::unique_ptr<PortalSettingEntry> darkModeEntry_;
+    std::unique_ptr<PortalSettingEntry> accentColorEntry_;
 #endif
 
     std::vector<std::unique_ptr<HandlerTableEntry<EventHandler>>>
         eventHandlers_;
+    std::vector<std::unique_ptr<HandlerTableEntry<EventHandler>>>
+        persistentEventHandlers_;
     std::unique_ptr<HandlerTableEntryBase> sniHandler_;
 
     std::unordered_map<std::string, std::unique_ptr<UIInterface>> uis_;
