@@ -6,6 +6,7 @@
  */
 #include "waylandinputwindow.h"
 #include <cstddef>
+#include "fcitx/misc_p.h"
 #include "common.h"
 #include "waylandim_public.h"
 #include "waylandui.h"
@@ -13,6 +14,7 @@
 #include "wl_compositor.h"
 #include "wl_region.h"
 #include "wp_fractional_scale_manager_v1.h"
+#include "zwp_input_method_v2.h"
 #include "zwp_input_panel_v1.h"
 #include "zwp_input_popup_surface_v2.h"
 
@@ -139,8 +141,9 @@ void WaylandInputWindow::update(fcitx::InputContext *ic) {
     const auto oldVisible = visible();
     auto [width, height] = InputWindow::update(ic);
     CLASSICUI_DEBUG() << "Wayland Input Window visible:" << visible()
-                      << " for IC program:" << ic->program()
-                      << " frontend:" << ic->frontend();
+                      << " for IC program:"
+                      << (ic ? ic->program() : std::string("-")) << " frontend:"
+                      << (ic ? ic->frontendName() : std::string("-"));
     if (!oldVisible && !visible()) {
         CLASSICUI_DEBUG() << "Wayland Input Window has been hidden.";
         return;
@@ -162,7 +165,7 @@ void WaylandInputWindow::update(fcitx::InputContext *ic) {
     CLASSICUI_DEBUG()
         << "Wayland Input Window is visible, ensure surface is created.";
     initPanel();
-    if (ic->frontend() == std::string_view("wayland_v2")) {
+    if (ic->frontendName() == "wayland_v2") {
         if (!panelSurfaceV2_ || ic != v2IC_.get()) {
             auto *waylandim = ui_->parent()->waylandim();
             if (!waylandim) {
@@ -183,7 +186,7 @@ void WaylandInputWindow::update(fcitx::InputContext *ic) {
             panelSurfaceV2_.reset();
             panelSurfaceV2_.reset(im->getInputPopupSurface(window_->surface()));
         }
-    } else if (ic->frontend() == std::string_view("wayland")) {
+    } else if (ic->frontendName() == "wayland") {
         auto panel = ui_->display()->getGlobal<wayland::ZwpInputPanelV1>();
         if (!panel) {
             return;
@@ -206,7 +209,8 @@ void WaylandInputWindow::update(fcitx::InputContext *ic) {
 
     if (auto *surface = window_->prerender()) {
         cairo_t *c = cairo_create(surface);
-        paint(c, width, height, window_->bufferScale());
+        paint(c, width, height,
+              window_->bufferScale() / WaylandWindow::ScaleDominatorF);
         cairo_destroy(c);
         window_->render();
     }
@@ -220,7 +224,8 @@ void WaylandInputWindow::repaint() {
 
     if (auto *surface = window_->prerender()) {
         cairo_t *c = cairo_create(surface);
-        paint(c, window_->width(), window_->height(), window_->bufferScale());
+        paint(c, window_->width(), window_->height(),
+              window_->bufferScale() / WaylandWindow::ScaleDominatorF);
         cairo_destroy(c);
         window_->render();
     }
