@@ -13,6 +13,7 @@
 #include "display.h"
 #include "org_kde_kwin_blur.h"
 #include "org_kde_kwin_blur_manager.h"
+#include "waylandcursortheme.h"
 #include "waylandinputwindow.h"
 #include "waylandshmwindow.h"
 #include "wl_compositor.h"
@@ -52,7 +53,8 @@ WaylandUI::WaylandUI(ClassicUI *parent, const std::string &name,
             } else if (name == wayland::WlSeat::interface) {
                 auto seat = display_->getGlobal<wayland::WlSeat>();
                 if (seat) {
-                    pointer_ = std::make_unique<WaylandPointer>(seat.get());
+                    pointer_ =
+                        std::make_unique<WaylandPointer>(this, seat.get());
                 }
             } else if (name == wayland::OrgKdeKwinBlurManager::interface) {
                 if (inputWindow_) {
@@ -85,9 +87,8 @@ WaylandUI::WaylandUI(ClassicUI *parent, const std::string &name,
         });
     auto seat = display_->getGlobal<wayland::WlSeat>();
     if (seat) {
-        pointer_ = std::make_unique<WaylandPointer>(seat.get());
+        pointer_ = std::make_unique<WaylandPointer>(this, seat.get());
     }
-    display_->sync();
     setupInputWindow();
 }
 
@@ -98,21 +99,24 @@ void WaylandUI::update(UserInterfaceComponent component,
     if (inputWindow_ && component == UserInterfaceComponent::InputPanel) {
         CLASSICUI_DEBUG() << "Update Wayland Input Window";
         inputWindow_->update(inputContext);
-        display_->flush();
     }
 }
 
-void WaylandUI::suspend() { inputWindow_.reset(); }
+void WaylandUI::suspend() {
+    if (!inputWindow_) {
+        return;
+    }
+    inputWindow_->update(nullptr);
+}
 
 void WaylandUI::resume() {
     CLASSICUI_DEBUG() << "Resume WaylandUI display name:" << display_;
-    setupInputWindow();
     CLASSICUI_DEBUG() << "Wayland Input window is initialized:"
                       << !!inputWindow_;
 }
 
 void WaylandUI::setupInputWindow() {
-    if (parent_->suspended() || inputWindow_) {
+    if (inputWindow_) {
         return;
     }
 
@@ -124,6 +128,8 @@ void WaylandUI::setupInputWindow() {
     if (!display_->getGlobal<wayland::WlCompositor>()) {
         return;
     }
+
+    cursorTheme_ = std::make_unique<WaylandCursorTheme>(this);
     inputWindow_ = std::make_unique<WaylandInputWindow>(this);
     inputWindow_->initPanel();
     inputWindow_->setBlurManager(
