@@ -69,7 +69,6 @@ bool XCBMenu::filterEvent(xcb_generic_event_t *event) {
                 hideChilds();
                 hide();
                 hideParents();
-                xcb_flush(ui_->connection());
             }
             return true;
         }
@@ -169,10 +168,13 @@ void XCBMenu::handleButtonPress(int eventX, int eventY) {
         // want to make ic has focus.
         activateTimer_ = ui_->parent()->instance()->eventLoop().addTimeEvent(
             CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 30000, 0,
-            [this, icRef, id](EventSourceTime *, uint64_t) {
+            [this, that = this->watch(), icRef, id](EventSourceTime *,
+                                                    uint64_t) {
+                if (!that.isValid()) {
+                    return true;
+                }
                 // FCITX_INFO() << "Timer Triggered";
                 if (auto *ic = icRef.get()) {
-
                     auto *action = ui_->parent()
                                        ->instance()
                                        ->userInterfaceManager()
@@ -233,7 +235,6 @@ void XCBMenu::hideAll() {
     hideParents();
     hide();
     hideChilds();
-    xcb_flush(ui_->connection());
 }
 
 bool XCBMenu::childHasMouse() const {
@@ -282,7 +283,6 @@ void XCBMenu::hideTillMenuHasMouseOrTopLevelHelper() {
     if (parent_.isNull() || hasMouse_) {
         update();
         setFocus();
-        xcb_flush(ui_->connection());
         return;
     }
     auto *parent = parent_.get();
@@ -305,19 +305,19 @@ void XCBMenu::setHoveredIndex(int idx) {
     pool_->setPopupMenuTimer(
         ui_->parent()->instance()->eventLoop().addTimeEvent(
             CLOCK_MONOTONIC, now(CLOCK_MONOTONIC) + 300000, 0,
-            [this](EventSourceTime *, uint64_t) {
+            [this, that = this->watch()](EventSourceTime *, uint64_t) {
+                if (!that.isValid()) {
+                    return true;
+                }
                 do {
                     // FCITX_INFO() << this << " in timer";
                     if (hoveredIndex_ >= 0 && subMenuIndex_ == hoveredIndex_) {
                         // Mouse is on same menu item.
-                        if (auto *child = child_.get()) {
-                            child->hideChilds();
-                            xcb_flush(ui_->connection());
-                        }
                         break;
                     }
 
                     if (hoveredIndex_ >= 0) {
+                        setFocus();
                         // FCITX_INFO() << this << " in timer branch 1";
                         // The current subMenu anyway is not the hovered one.
                         hideChilds();
@@ -336,7 +336,8 @@ void XCBMenu::setHoveredIndex(int idx) {
                             // FCITX_INFO() << this << " in timer show submenu "
                             // << newMenu;
                             newMenu->show(
-                                item.first->region_.translated(x_, y_));
+                                item.first->region_.translated(x_, y_),
+                                ConstrainAdjustment::Slide);
                         }
                     } else {
                         /// FCITX_INFO() << this << " in timer branch 2";
@@ -345,7 +346,6 @@ void XCBMenu::setHoveredIndex(int idx) {
                         hideTillMenuHasMouseOrTopLevel();
                     }
                     update();
-                    xcb_flush(ui_->connection());
                 } while (0);
                 pool_->setPopupMenuTimer(nullptr);
                 return true;
@@ -640,7 +640,7 @@ void XCBMenu::setFocus() {
                         XCB_CURRENT_TIME);
 }
 
-void XCBMenu::show(Rect rect) {
+void XCBMenu::show(Rect rect, ConstrainAdjustment adjustY) {
     // FCITX_INFO() << this << " show() " << hoveredIndex_;
     if (visible_) {
         return;
@@ -665,29 +665,27 @@ void XCBMenu::show(Rect rect) {
     x = x + rect.width();
 
     if (closestScreen) {
-        int newX, newY;
 
         if (x + width() > closestScreen->right()) {
-            newX = rect.left() - width();
-        } else {
-            newX = x;
+            x = rect.left() - width();
         }
 
+        switch (adjustY) {
+        case ConstrainAdjustment::Slide:
+            if (y + height() > closestScreen->bottom()) {
+                y = closestScreen->bottom() - height();
+            }
+            break;
+        case ConstrainAdjustment::Flip:
+            if (y + height() > closestScreen->bottom()) {
+                y = rect.top() - height();
+            }
+            break;
+        };
+
         if (y < closestScreen->top()) {
-            newY = closestScreen->top();
-        } else {
-            newY = y;
+            y = closestScreen->top();
         }
-
-        if (newY + height() > closestScreen->bottom()) {
-            if (newY > closestScreen->bottom()) {
-                newY = closestScreen->bottom() - height();
-            } else { /* better position the window */
-                newY = newY - height();
-            }
-        }
-        x = newX;
-        y = newY;
     }
 
     xcb_params_configure_window_t wc;
@@ -704,7 +702,6 @@ void XCBMenu::show(Rect rect) {
     if (parent_.isNull()) {
         ui_->grabPointer(this);
     }
-    xcb_flush(ui_->connection());
     x_ = x;
     y_ = y;
 }
