@@ -99,7 +99,8 @@ bool XCBTrayWindow::filterEvent(xcb_generic_event_t *event) {
                 XCBMenu *menu = menuPool_.requestMenu(ui_, &menu_, nullptr);
                 menu->show(Rect()
                                .setPosition(press->root_x, press->root_y)
-                               .setSize(1, 1));
+                               .setSize(1, 1),
+                           ConstrainAdjustment::Flip);
             } else if (press->detail == XCB_BUTTON_INDEX_1) {
                 ui_->parent()->instance()->toggle();
             }
@@ -246,7 +247,6 @@ void XCBTrayWindow::sendTrayOpcode(long message, long data1, long data2,
 
     xcb_send_event(ui_->connection(), false, dockWindow_,
                    XCB_EVENT_MASK_NO_EVENT, reinterpret_cast<char *>(&ev));
-    xcb_flush(ui_->connection());
 }
 
 xcb_visualid_t XCBTrayWindow::trayVisual() {
@@ -330,7 +330,6 @@ void XCBTrayWindow::postCreateWindow() {
             ui_->connection(), wid_,
             XCB_CW_BACKING_PIXEL | XCB_CW_BORDER_PIXEL | XCB_CW_BACK_PIXMAP,
             &list);
-        xcb_flush(ui_->connection());
     }
 }
 
@@ -401,7 +400,6 @@ void XCBTrayWindow::render() {
     cairo_paint(cr);
     cairo_destroy(cr);
     cairo_surface_flush(surface_.get());
-    xcb_flush(ui_->connection());
     CLASSICUI_DEBUG() << "Render";
 }
 
@@ -504,8 +502,9 @@ void XCBTrayWindow::updateInputMethodMenu() {
         auto &inputMethodAction = inputMethodActions_.back();
         inputMethodAction.setShortText(entry->name());
         inputMethodAction.connect<SimpleAction::Activated>(
-            [this, imName](InputContext *) {
-                ui_->parent()->instance()->setCurrentInputMethod(imName);
+            [this, imName](InputContext *ic) {
+                ui_->parent()->instance()->setCurrentInputMethod(ic, imName,
+                                                                 false);
             });
         inputMethodAction.setCheckable(true);
         inputMethodAction.setChecked(
