@@ -6,7 +6,6 @@
  */
 
 #include "kimpanel.h"
-#include <fcitx/inputmethodengine.h>
 #include "fcitx-utils/dbus/objectvtable.h"
 #include "fcitx-utils/dbus/servicewatcher.h"
 #include "fcitx-utils/i18n.h"
@@ -16,6 +15,7 @@
 #include "fcitx/action.h"
 #include "fcitx/addonmanager.h"
 #include "fcitx/inputcontext.h"
+#include "fcitx/inputmethodengine.h"
 #include "fcitx/inputmethodentry.h"
 #include "fcitx/inputmethodmanager.h"
 #include "fcitx/instance.h"
@@ -24,6 +24,9 @@
 #include "fcitx/userinterfacemanager.h"
 #include "dbus_public.h"
 
+#define FCITX_NO_XCB
+#include "xcb_public.h"
+
 namespace fcitx {
 
 enum class CursorRectMethod {
@@ -262,8 +265,26 @@ void Kimpanel::resume() {
 
 void Kimpanel::update(UserInterfaceComponent component,
                       InputContext *inputContext) {
+    if (!inputContext->hasFocus() && inputContext != lastInputContext_.get() &&
+        inputContext != instance_->mostRecentInputContext()) {
+        return;
+    }
     if (component == UserInterfaceComponent::InputPanel) {
-        updateInputPanel(inputContext);
+        if (classicui() && isKDE() &&
+            (stringutils::startsWith(inputContext->frontendName(), "wayland") ||
+             (xcb() &&
+              stringutils::startsWith(inputContext->display(), "x11:") &&
+              xcb()->call<IXCBModule::isXWayland>(
+                  inputContext->display().substr(4))))) {
+            proxy_->showAux(false);
+            proxy_->showPreedit(false);
+            proxy_->showLookupTable(false);
+            static_cast<UserInterface *>(classicui())
+                ->update(component, inputContext);
+            lastInputContext_ = inputContext->watch();
+        } else {
+            updateInputPanel(inputContext);
+        }
     } else if (component == UserInterfaceComponent::StatusArea) {
         registerAllProperties(inputContext);
     }
