@@ -49,9 +49,16 @@ int main() {
     char arg0[] = "testinstance";
     char arg1[] = "--disable=all";
     char arg2[] = "--enable=testim";
-    char *argv[] = {arg0, arg1, arg2};
+    char arg3[] = "--option=name1=opt1a:opt1b,name2=opt2a:opt2b";
+    char *argv[] = {arg0, arg1, arg2, arg3};
     Instance instance(FCITX_ARRAY_SIZE(argv), argv);
     instance.addonManager().registerDefaultLoader(nullptr);
+    FCITX_ASSERT(instance.addonManager().addonOptions("name1") ==
+                 std::vector<std::string>{"opt1a", "opt1b"});
+    FCITX_ASSERT(instance.addonManager().addonOptions("name2") ==
+                 std::vector<std::string>{"opt2a", "opt2b"});
+    FCITX_ASSERT(instance.addonManager().addonOptions("name3") ==
+                 std::vector<std::string>{});
     EventDispatcher dispatcher;
     dispatcher.attach(&instance.eventLoop());
     testCheckUpdate(&dispatcher, &instance);
